@@ -4,10 +4,10 @@ from dataclasses import dataclass, field
 from enum import Enum, IntEnum, auto
 import os
 import re
-from typing import Iterable, List, Optional, Union
+from typing import Iterable, List, Optional, NamedTuple, Union
 import warnings
 
-from avlwrapper import VERSION
+from avlwrapper import VERSION, logger
 
 
 class InputError(ValueError):
@@ -23,12 +23,40 @@ class InputError(ValueError):
 class Input(ABC):
     @classmethod
     def from_lines(cls, lines_in: List[str]):
+        # remove commented lines
+        lines = list(filter(line_has_no_comment, lines_in))
+
+        # remove empty lines
+        lines = list(filter(line_is_not_empty, lines))
+
+        # remove lines with only "-" and spaces
+        lines = list(filter(line_is_not_separator, lines))
+
+        return cls._from_lines(lines)
+
+    @classmethod
+    def _from_lines(cls, lines_in: List[str]):
         """
         To be implemented by subclasses. This method creates an instance
         from a list of strings from the input file representing the object
         """
         raise NotImplementedError
 
+    @classmethod
+    def from_file(cls, filename):
+        """
+        Generates Model from AVL input file
+        """
+
+        if not os.path.isabs(filename):
+            filename = os.path.abspath(filename)
+
+        # read file to lines
+        with open(filename, "rt") as fp:
+            lines = fp.readlines()
+
+        return cls.from_lines(lines)
+
 
 class ModelInput(Input, ABC):
     @classmethod
@@ -62,22 +90,34 @@ class ModelInput(Input, ABC):
             if not param.attr_type == AttrType.list and param.attr in kwargs:
                 raise ValueError(f"Only one {token} is allowed")
 
+            # parse values
             if param.cls is None:
-                if param.attr_type == AttrType.scalar:
-                    value = float(data[1].strip())
+                if param.attr_type == AttrType.float:
+                    kwargs[param.attr] = float(data[1].strip())
+                elif param.attr_type == AttrType.int:
+                    value = data[1].strip()
+                    try:
+                        value = int(value)
+                    except ValueError:
+                        value = float(int(value))
+                        logger.warning(f"Float converted to int: {param.name}")
+                    finally:
+                        kwargs[param.attr] = value
+
                 elif param.attr_type == AttrType.boolean:
-                    value = True
+                    kwargs[param.attr] = True
                 elif param.attr_type == AttrType.vector:
                     vals = [float(s) for s in data[1].split()]
                     if len(vals) != 3:
                         raise InputError(data)
-                    value = Vector(*vals)
+                    kwargs[param.attr] = Vector(*vals)
                 else:
                     assert False
-                kwargs[param.attr] = value
+
+            # instantiate classes
             else:
                 obj = param.cls.from_lines(data)
-                if param.attr_type == AttrType.scalar:
+                if param.attr_type == AttrType.single:
                     kwargs[param.attr] = obj
                 elif param.attr_type == AttrType.list:
                     kwargs[param.attr].append(obj)
@@ -86,8 +126,32 @@ class ModelInput(Input, ABC):
         return kwargs
 
 
-Spacial = namedtuple("Spacial", ["x", "y", "z"])
-Spacial.__str__ = lambda self: f"{self.x} {self.y} {self.z}"
+class Spacial(NamedTuple):
+    x: float
+    y: float
+    z: float
+
+    def __str__(self):
+        return f"{self.x} {self.y} {self.z}"
+
+    def __add__(self, other):
+        kwargs = {}
+        for attr in ["x", "y", "z"]:
+            kwargs[attr] = getattr(self, attr) + getattr(other, attr)
+        return Spacial(**kwargs)
+
+    def __mul__(self, other):
+        kwargs = {}
+        for attr in ["x", "y", "z"]:
+            kwargs[attr] = getattr(self, attr) * other
+        return Spacial(**kwargs)
+
+    def __truediv__(self, other):
+        kwargs = {}
+        for attr in ["x", "y", "z"]:
+            kwargs[attr] = getattr(self, attr) / other
+        return Spacial(**kwargs)
+
 
 Point = Spacial
 Vector = Spacial
@@ -98,12 +162,28 @@ class IntStrEnum(IntEnum):
         return str(self.value)
 
 
-class Spacing(IntStrEnum):
+class FloatStrEnum(IntEnum):
+    def __str__(self):
+        return str(float(self.value))
+
+
+class Spacing(FloatStrEnum):
     sine = 2
     cosine = 1
     equal = 0
     neg_sine = -2
 
+    @classmethod
+    def parse(cls, val, force=False):
+        try:
+            return cls(val)
+        except ValueError:
+            if force:
+                return cls(round(val))
+            else:
+                logger.info(f"{val} not converted to Spacing")
+                return val
+
 
 class Symmetry(IntStrEnum):
     none = 0
@@ -192,7 +272,7 @@ class NacaAirfoil(Airfoil, _NacaAirfoil):
         return "naca"
 
     @classmethod
-    def from_lines(cls, lines_in):
+    def _from_lines(cls, lines_in):
         if len(lines_in) != 2:
             raise InputError(lines_in)
         x1, x2 = cls.read_x1_x2(lines_in[0])
@@ -218,8 +298,11 @@ class DataAirfoil(Airfoil, _DataAirfoil):
         return "airfoil"
 
     @classmethod
-    def from_lines(cls, lines_in):
+    def _from_lines(cls, lines_in):
         x1, x2 = cls.read_x1_x2(lines_in[0])
+        # The first line contains the "AIRFOIL" keyword and optionally x1 and x2 but no x/z data, so it must be removed
+        # prior to parsing the x/z data
+        del lines_in[0]
         xs, zs = zip(
             *[
                 (float(line[0]), float(line[1]))
@@ -244,7 +327,7 @@ class FileAirfoil(Airfoil, _FileAirfoil):
         return "afile"
 
     @classmethod
-    def from_lines(cls, lines_in):
+    def _from_lines(cls, lines_in):
         if len(lines_in) != 2:
             raise InputError(lines_in)
         x1, x2 = cls.read_x1_x2(lines_in[0])
@@ -296,7 +379,7 @@ class Control(ModelInput):
         )
 
     @classmethod
-    def from_lines(cls, lines_in):
+    def _from_lines(cls, lines_in):
         if len(lines_in) != 2:
             raise InputError(lines_in)
         params = lines_in[1].split()
@@ -335,7 +418,7 @@ class DesignVar(ModelInput):
         return f"DESIGN\n#Name Weight\n{self.name} {self.weight}\n"
 
     @classmethod
-    def from_lines(cls, lines_in):
+    def _from_lines(cls, lines_in):
         if len(lines_in) != 2:
             raise InputError(lines_in)
         params = lines_in[1].split()
@@ -369,7 +452,7 @@ class ProfileDrag(ModelInput):
         return header + body + "\n"
 
     @classmethod
-    def from_lines(cls, lines_in):
+    def _from_lines(cls, lines_in):
         if len(lines_in) != 2:
             raise InputError(lines_in)
         params = [float(s) for s in lines_in[1].split()]
@@ -440,7 +523,7 @@ class Section(ModelInput):
         return self._header_str + self._body_str
 
     @classmethod
-    def from_lines(cls, lines_in):
+    def _from_lines(cls, lines_in):
         # first 2 lines contain section data, rest contains airfoils, etc.
         header_lines = lines_in[:2]
         body_lines = lines_in[2:]
@@ -454,8 +537,8 @@ class Section(ModelInput):
             "angle": params[4],
         }
         if len(params) == 7:
-            kwargs["n_spanwise"] = params[5]
-            kwargs["span_spacing"] = params[6]
+            kwargs["n_spanwise"] = int(params[5])
+            kwargs["span_spacing"] = Spacing.parse(params[6])
 
         kwargs.update(cls.parse_lines(body_lines))
 
@@ -564,7 +647,7 @@ class Surface(ModelInput):
         return tokens
 
     @classmethod
-    def from_lines(cls, lines_in):
+    def _from_lines(cls, lines_in):
         # first 3 lines contain name and surface parameters
         header_lines = lines_in[:3]
         body_lines = lines_in[3:]
@@ -577,11 +660,11 @@ class Surface(ModelInput):
         kwargs = {
             "name": name,
             "n_chordwise": int(params[0]),
-            "chord_spacing": params[1],
+            "chord_spacing": Spacing.parse(params[1]),
         }
         if len(params) == 4:
             kwargs["n_spanwise"] = int(params[2])
-            kwargs["span_spacing"] = params[3]
+            kwargs["span_spacing"] = Spacing.parse(params[3])
 
         kwargs.update(cls.parse_lines(body_lines))
 
@@ -625,7 +708,7 @@ class Body(ModelInput):
         return s
 
     @classmethod
-    def from_lines(cls, lines_in):
+    def _from_lines(cls, lines_in):
         # first 3 lines contain name and body parameters
         header_lines = lines_in[:3]
         body_lines = lines_in[3:]
@@ -634,7 +717,11 @@ class Body(ModelInput):
         params = line_to_floats(header_lines[2])
         if len(params) != 2:
             InputError(header_lines)
-        kwargs = {"name": name, "n_body": int(params[0]), "body_spacing": params[1]}
+        kwargs = {
+            "name": name,
+            "n_body": int(params[0]),
+            "body_spacing": Spacing.parse(params[1]),
+        }
 
         kwargs.update(cls.parse_lines(body_lines))
 
@@ -672,7 +759,7 @@ class Aircraft(ModelInput):
     y_symmetry: Symmetry = Symmetry.none
     z_symmetry: Symmetry = Symmetry.none
     z_symmetry_plane: float = 0.0
-    
+
     _from_file: Optional[str] = None
 
     def __str__(self):
@@ -693,7 +780,7 @@ class Aircraft(ModelInput):
         )
 
     @classmethod
-    def from_lines(cls, lines_in, file_path=None):
+    def _from_lines(cls, lines_in):
         # first 5 or 6 lines contain name and parameters
         keywords = [k[:5] for k in list(KEYWORDS[Aircraft].keys())]
         if any([lines_in[5].startswith(key) for key in keywords]):
@@ -706,7 +793,6 @@ class Aircraft(ModelInput):
         kwargs = {
             "name": header_lines[0].strip(),
             "mach": float(header_lines[1].strip()),
-            "_from_file": file_path
         }
 
         symmetry_params = line_to_floats(header_lines[2])
@@ -727,24 +813,9 @@ class Aircraft(ModelInput):
 
     @classmethod
     def from_file(cls, filename):
-        """
-        Generates Aircraft from AVL input file
-        """
-        
-        if not os.path.isabs(filename):
-            filename = os.path.abspath(filename)
-
-        # read file to lines
-        with open(filename, "rt") as fp:
-            lines = fp.readlines()
-
-        # remove commented lines
-        lines = list(filter(line_has_no_comment, lines))
-
-        # remove empty lines
-        lines = list(filter(line_is_empty, lines))
-
-        return cls.from_lines(lines, filename)
+        obj = super().from_file(filename)
+        obj._from_file = filename
+        return obj
 
     @property
     def external_files(self):
@@ -755,7 +826,7 @@ class Aircraft(ModelInput):
                     files.add(section.airfoil.filename)
         for body in self.bodies:
             files.add(body.body_section.filename)
-        
+
         if self._from_file is None:
             af_dir = os.getcwd()
         else:
@@ -764,15 +835,6 @@ class Aircraft(ModelInput):
         return files
 
 
-class Geometry(Aircraft):
-    def __init__(self, *args, **kwargs):
-        warnings.warn(
-            "Use avlwrapper.Aircraft, this class will be removed in a future version",
-            DeprecationWarning,
-        )
-        super().__init__(*args, **kwargs)
-
-
 @dataclass
 class Parameter(Input):
     """Parameter used in the case definition
@@ -792,7 +854,7 @@ class Parameter(Input):
             self.setting = self.name
 
     @classmethod
-    def from_lines(cls, lines_in: List[str]):
+    def _from_lines(cls, lines_in: List[str]):
         if len(lines_in) != 1:
             raise InputError(lines_in)
         name, setting, value_str = (
@@ -814,7 +876,7 @@ class State(Input):
     unit: str = ""
 
     @classmethod
-    def from_lines(cls, lines_in: List[str]):
+    def _from_lines(cls, lines_in: List[str]):
         if len(lines_in) != 1:
             raise InputError(lines_in)
         params = multi_split(lines_in[0], "=", " ")
@@ -855,8 +917,11 @@ class Case(Input):
         "CL",
         "CY",
         "Cl",
+        "Cl roll mom",
         "Cm",
+        "Cm pitchmom",
         "Cn",
+        "Cn yaw  mom",
     }
 
     CASE_STATES = {
@@ -932,6 +997,8 @@ class Case(Input):
             # if a parameter object is given, add to the dict
             if isinstance(value, Parameter):
                 self.parameters[key] = value
+                if key not in self.CASE_PARAMETERS:
+                    self.controls.append(key)
             else:
                 # if the key is an existing case parameter, set the value
                 if key in self.CASE_PARAMETERS:
@@ -947,12 +1014,12 @@ class Case(Input):
                     self.parameters[param_str] = Parameter(name=param_str, value=value)
 
     @classmethod
-    def from_lines(cls, lines_in: List[str]):
+    def _from_lines(cls, lines_in: List[str]):
         """
         Create a Case instance from lines in case-file format
         """
         # get case number and title
-        re_str = r"(?<=Run case)\s*(\d+)\s*:\s*(\w+)"
+        re_str = r"(?<=Run case)\s*(\d+)\s*:\s*(\S+)"
         match = re.search(re_str, lines_in[0], re.IGNORECASE)
         if match is not None:
             number = int(match.group(1))
@@ -962,15 +1029,21 @@ class Case(Input):
             number = 1
             name = "unknown"
 
-        params = []
+        params = {}
+        states = []
         for line in lines_in[1:]:
             if "->" in line:
                 param = Parameter.from_lines([line.strip()])
+                try:
+                    key = cls._get_parameter_key_by_name(param.name)
+                except LookupError:
+                    key = param.name
+                params[key] = param
             else:
-                param = State.from_lines([line.strip()])
-            params.append(param)
+                state = State.from_lines([line.strip()])
+                states.append(state)
 
-        return cls(name, *params, number=number)
+        return cls(name, *states, number=number, **params)
 
     def _set_default_parameters(self):
         # parameters default to 0.0
@@ -1002,14 +1075,16 @@ class Case(Input):
         self._check_parameters()
         self._check_states()
 
-    def _get_parameter_key_by_name(self, name):
-        for key, value in self.CASE_PARAMETERS.items():
+    @classmethod
+    def _get_parameter_key_by_name(cls, name):
+        for key, value in cls.CASE_PARAMETERS.items():
             if value.lower().strip() == name.lower().strip():
                 return key
         raise LookupError(f"{name} not found")
 
-    def _get_state_key_by_name(self, name):
-        for key, value in self.CASE_STATES.items():
+    @classmethod
+    def _get_state_key_by_name(cls, name):
+        for key, value in cls.CASE_STATES.items():
             if value[0].lower().strip() == name.lower().strip():
                 return key
         raise LookupError(f"{name} not found")
@@ -1038,7 +1113,7 @@ def read_case_file(filename):
         lines = fp.readlines()
 
     # remove empty lines
-    lines = list(filter(line_is_empty, lines))
+    lines = list(filter(line_is_not_empty, lines))
 
     # remove separator lines
     lines = list(filter(lambda line: not line.strip().startswith("-"), lines))
@@ -1075,7 +1150,7 @@ def multi_split(str_in, *seps):
     return str_lst
 
 
-def line_is_empty(line):
+def line_is_not_empty(line):
     return line.strip() != ""
 
 
@@ -1083,14 +1158,20 @@ def line_has_no_comment(line):
     return not (line.startswith("!") or line.startswith("#"))
 
 
+def line_is_not_separator(line):
+    return set(line.strip()) != {"-"}
+
+
 ParameterType = namedtuple("ParameterType", ["cls", "attr", "attr_type"])
 
 
 class AttrType(Enum):
-    scalar = auto()
+    single = auto()
     list = auto()
     vector = auto()
     boolean = auto()
+    float = auto()
+    int = auto()
 
 
 # Mapping of keywords to avlwrapper classes
@@ -1098,33 +1179,33 @@ PT = ParameterType  # Short-hand
 KEYWORDS = {
     Aircraft: {
         "SURFACE": PT(Surface, "surfaces", AttrType.list),
-        "BODY": PT(Surface, "bodies", AttrType.list),
+        "BODY": PT(Body, "bodies", AttrType.list),
     },
     Surface: {
-        "COMPONENT": PT(None, "component", AttrType.scalar),
-        "INDEX": PT(None, "component", AttrType.scalar),
-        "YDUPLICATE": PT(None, "y_duplicate", AttrType.scalar),
+        "COMPONENT": PT(None, "component", AttrType.int),
+        "INDEX": PT(None, "component", AttrType.int),
+        "YDUPLICATE": PT(None, "y_duplicate", AttrType.float),
         "SCALE": PT(None, "scaling", AttrType.vector),
         "TRANSLATE": PT(None, "translation", AttrType.vector),
-        "ANGLE": PT(None, "angle", AttrType.scalar),
+        "ANGLE": PT(None, "angle", AttrType.float),
         "NOWAKE": PT(None, "no_wake", AttrType.boolean),
         "NOALBE": PT(None, "fixed", AttrType.boolean),
         "NOLOAD": PT(None, "no_loads", AttrType.boolean),
-        "CDCL": PT(ProfileDrag, "profile_drag", AttrType.scalar),
+        "CDCL": PT(ProfileDrag, "profile_drag", AttrType.float),
         "SECTION": PT(Section, "sections", AttrType.list),
     },
     Body: {
-        "YDUPLICATE": PT(None, "y_duplicate", AttrType.scalar),
+        "YDUPLICATE": PT(None, "y_duplicate", AttrType.float),
         "SCALE": PT(None, "scaling", AttrType.vector),
         "TRANSLATE": PT(None, "translation", AttrType.vector),
-        "BFILE": PT(BodyProfile, "body_section", AttrType.scalar),
+        "BFILE": PT(BodyProfile, "body_section", AttrType.single),
     },
     Section: {
-        "NACA": PT(NacaAirfoil, "airfoil", AttrType.scalar),
-        "AIRFOIL": PT(DataAirfoil, "airfoil", AttrType.scalar),
-        "CLAF": PT(None, "cl_alpha_scaling", AttrType.scalar),
-        "CDCL": PT(ProfileDrag, "profile_drag", AttrType.scalar),
-        "AFILE": PT(FileAirfoil, "airfoil", AttrType.scalar),
+        "NACA": PT(NacaAirfoil, "airfoil", AttrType.single),
+        "AIRFOIL": PT(DataAirfoil, "airfoil", AttrType.single),
+        "CLAF": PT(None, "cl_alpha_scaling", AttrType.float),
+        "CDCL": PT(ProfileDrag, "profile_drag", AttrType.float),
+        "AFILE": PT(FileAirfoil, "airfoil", AttrType.single),
         "CONTROL": PT(Control, "controls", AttrType.list),
     },
 }
