@@ -2,6 +2,8 @@ import os.path
 import re
 import warnings
 
+from avlwrapper import logger
+
 
 # pattern to match a floating point number with:
 # optional leading '+' or '-'
@@ -10,7 +12,7 @@ import warnings
 # (optionally) an exponent
 #   - with lowercase 'e' or uppercase 'E'
 #   - (optionally) with a '+' or '-' before the power
-FLOATING_POINT_PATTERN = r'[+-]?\d+(?:\.\d+)?(?:[eE][+-]?\d+)?'
+FLOATING_POINT_PATTERN = r"[+-]?\d+(?:\.\d+)?(?:[eE][+-]?\d+)?"
 
 
 class FileReader:
@@ -28,7 +30,9 @@ class FileReader:
     def get_vars(lines):
         # Search for "key = value" tuples and store in a dictionary
         result = dict()
-        for name, value in re.findall(fr"(\S+)\s+=\s+({FLOATING_POINT_PATTERN})", "".join(lines)):
+        for name, value in re.findall(
+            rf"(\S+)\s+=\s*({FLOATING_POINT_PATTERN})", "".join(lines)
+        ):
             result[name] = float(value)
         return result
 
@@ -61,11 +65,11 @@ class FileReader:
 
     @staticmethod
     def get_line_values(data_line):
-        data_list = re.findall(rf"({FLOATING_POINT_PATTERN}|\*{8})", data_line)
+        data_list = re.findall(rf"({FLOATING_POINT_PATTERN}|\*+)", data_line)
         values = []
         raised_warning = False
         for val in data_list:
-            if val == "*" * 8:
+            if "*" in val:
                 values.append(float("nan"))
                 if not raised_warning:
                     msg = (
@@ -73,7 +77,7 @@ class FileReader:
                         "Most likely the value contained more characters "
                         "than the AVL output formatter supports:\n"
                     )
-                    warnings.warn(msg + data_line)
+                    logger.warning(msg + data_line)
                 raised_warning = True
             else:
                 values.append(float(val))
@@ -131,7 +135,7 @@ class SurfaceFileReader(FileReader):
             # ignore first column
             line_data = line_data[1:]
 
-            name = re.findall(r"[^-\dE.\s+\*](.+)\n", line)[0].strip()
+            name = re.findall(r"(\D*)$", line)[0].strip()
 
             if len(line_data) < len(header):
                 raise ValueError("Incorrect table format")
@@ -194,6 +198,13 @@ class StripFileReader(FileReader):
                 # ignore first column
                 if ignore_first:
                     values = values[1:]
+
+                if len(values) < len(header):
+                    logger.warning("Table values missing. Replaced with NaN")
+                    values += [float("nan")] * (len(header) - len(values))
+                elif len(values) > len(header):
+                    raise ValueError("Incorrect table format")
+
                 for key, value in zip(header, values):
                     strip_results[result_name][key].append(value)
         return strip_results
@@ -212,7 +223,7 @@ class ElementFileReader(FileReader):
         data_tables = dict()
         for surface_name, surface_lines in list(surface_tables.items()):
             # tables split by strip
-            strip_tables = self.split_lines(surface_lines, r"Strip\s+#\s+(\d+)\s+")
+            strip_tables = self.split_lines(surface_lines, r"Strip\s+#\s*(\d+)\s+")
             header_re = r"(I\s+X\s+Y\s+Z)"
             data_tables[surface_name] = dict()
             for strip_name, strip_lines in list(strip_tables.items()):
@@ -275,7 +286,7 @@ class StabilityFileReader(FileReader):
             match = re.search(r"d\d+", key)
             if match is not None:
                 d = match.group(0)
-                name = controls[d]
+                name = "_" + controls[d]
                 new_key = re.sub(d, name, key)
                 new_dict[new_key] = new_dict[key]
                 new_dict.pop(key)
